#!/bin/sh

##########################################################################
# ifacc.cgi 
# version 0.3.7, 2005-02-09
# Jens Roesiger <jensr@coyotelinux.de>
# 
# Ipacc.cgi show the accounting of traffic over the external interface
# for every defined IP's in /etc/ipacc.conf and output in 
# Web Administrator
#
# Changelog
# ipacc.cgi 0.3.7
#  - include routine to edit ipacc.conf. Source is a part of editconf.cgi
#    now, changes at ipacc.conf are immediately applyed
#  - new option in /etc/ipacc.conf:
#    DISPLAY_FIRST_PAGE=daily
#    with this line, display at the first page of ipacc the daily traffic
#  - you can select the traffic at today
#
# ipacc.cgi 0.3.6
#  - "drop down box" to select accounting month
#  - display version at end of HTML output
#
# ipacc.cgi 0.3.5
#  - bugfix: display false traffic, traffic values in files are ok
#
# ipacc.cgi 0.3.4
#  - bugfix: display always zero traffic (zero values in files)
#  - bugfix: display false traffic, traffic values in files are ok 
#
# ipacc.cgi 0.3.3
#  - rename /usr/local/bin/accsum to ipaccsum. If accsum is a part of
#   ifacc and ipacc, accsum is exclude from backup.
#
# ipacc.cgi 0.3.2
#  - /etc/ipacc.conf is no longer in default ipacc.tgz. 
#    /etc/ipacc.conf is created at first start of ipacc.cgi
#   Don't overwrite if update of ipacc.tgz is available  
#  - fix error in calculation of accounting (to big values)
#  - correct sort the IP addresses (with 3 sed and one sort)
#  - ipacc.tgz include ipacc-dummy.txt in order to avoid errors with backup, 
#    if no ipacc-DATE.txt is created
#
# ipacc.cgi 0.3.1
#  - fix some problems after installtion and without ip's in /etc/ipacc.conf
#  - use selfcreated program accsum for calculate accounting information
#
# ipacc.cgi 0.3
#  - new option (link) to recreate accounting chain after edit /etc/ipacc.conf
#  - remove option OUTPUT=csv
#  - read the name of accounting chain from /etc/rc.d/pkgs/rc.ipacc
#  - rename accounting chain
#  - shell option 'quiet' for no output (for run with cron jobs)
#
# ipacc.cgi 0.2.3
#  - more reduced code
#  - read new accounting only if ACCMON is not indicated in URL (faster output)
#
# ipacc.cgi 0.2.2
#  - new output style, display traffic for each ip for indicated month
#  - diplay traffic for a single ip, if you indicated
#
# ipacc.cgi 0.2.1
#  - edit /etc/ipacc.conf with webbrowser
#
# ipacc.cgi 0.2
#  - read and write accounting information faster
#  - speedup HTML output
#  - show only accounting information for indicated day
#
# ipacc.cgi 0.1.1
#  - reduce quellcode
#
# ipacc.cgi 0.1
#  - create first version of ipacc (copy from ifacc and adapted)
#
##########################################################################

#set -x

# read coyote config file
. /etc/coyote/coyote.conf

# Source the Web Functions
. /var/http/web-functions

#####################################################################
print_HTML () {

	my_ACC_TEXT=${1};
	my_ACC_OUT=${2};
	my_ACC_IN=${3};
	my_ACC_SUM=${4};
	my_TD_BG_COLOR=${5};
	my_ACC_TEXT_LINK=${6};
	if [ "$ACC_CONVERT" = "1" ]; then
		# convert to k|M Byte	
		my_ACC_OUT=`$ACCSUM $my_ACC_OUT 0`;
		# convert to k|M Byte
		my_ACC_IN=`$ACCSUM $my_ACC_IN 0`;
		# convert to k|M Byte
		my_ACC_SUM=`$ACCSUM $my_ACC_SUM 0`;
	
		#echo "$my_ACC_IP $USE_ACC_IP<br>";
	
		if [ "$USE_ACC_IP" = "0" -a "$my_ACC_MON" = "$SHOW_ACC_MON" ]; then
			my_ACC_TEXT="*&nbsp;$my_ACC_TEXT";
		else
			my_ACC_TEXT="&nbsp;&nbsp;$my_ACC_TEXT";	
		fi	
	fi

	# HTML output
        echo "  <tr>";
        echo "   <td colspan=2 bgcolor=\"$my_TD_BG_COLOR\" align=center width=25%>";
        # my_ACC_TEXT display as link?
        if [ -z "$my_ACC_TEXT_LINK" ]; then
        	echo "<b>$my_ACC_TEXT</b></td>";
        else
                echo "<a href=ipacc.cgi?ACCMON=$ACC_MON&AMP;ACCIP=$ACC_IP><b>$my_ACC_TEXT</b></a></td>";
        fi
        echo -n "   <td colspan=2 bgcolor=\"$my_TD_BG_COLOR\" align=center width=25%>$my_ACC_OUT";
        echo "Byte</td>";
        echo -n "   <td colspan=2 bgcolor=\"$my_TD_BG_COLOR\" align=center width=25%>$my_ACC_IN";
        echo "Byte</td>";
        echo -n "   <td colspan=2 bgcolor=\"$my_TD_BG_COLOR\" align=center width=25%>$my_ACC_SUM";
        echo "Byte</td>";
        echo "  </tr>";
}

# end of functions
#####################################################################

#####################################################################
# beginn of variables
#

# version

VERSION="0.3.7"

# if cron for cron, no output
QUIET="0";
if [ "$1" = "quiet" ];then 
	QUIET="1"
fi

# the accounting interface
if [ "$INETTYPE" = "PPPOE" -o "$INETTYPE" = "PPP" ]; then
        IF_INET=ppp0
elif [ -z "$IF_INET" ] ; then
        IF_INET=eth1
fi

# HTML title
TITLE="Accounting - Coyote Linux";

# today is
DATE=`/bin/date +%Y%m%d-%H%M%S`;

# where is iptables
IPTABLES="/usr/sbin/iptables";

ACCSUM="/usr/local/bin/ipaccsum";

# the accounting files
ACC_DATE=`/bin/date +%Y%m%d`;
ACC_MON=`/bin/date +%Y%m`;
my_ACC_MON=$ACC_MON;		
ACC_FILE="/var/spool/accounting/ipacc-$ACC_DATE.txt";
ACC_FILE_OLD="/var/spool/accounting/old_ipacc-$ACC_DATE.txt";

# script to create accounting rules 
RC_IPACC="/etc/rc.d/pkgs/rc.ipacc";

# name of accounting chain
ACC_CHAIN=`grep "^ACC_CHAIN=" $RC_IPACC | cut -d '"' -f2`

# editmodus
EDIT_CONF=0;
if [ "$FORM_EDITCONF" = "1" ]; then
	EDIT_CONF=1;
fi

# name of config file
IPACC_CONF="/etc/ipacc.conf";
# create configfile if not exist
if [ ! -e "$IPACC_CONF" ]; then
cat >$IPACC_CONF << CLEOF
# config file for ipacc
# insert the IP addresses here
# ! only one IP per line !
# line with beginning # are comments
#
# if you want to display the daily traffic at the first page of ipacc, 
# remove the '#' at the next line
#DISPLAY_FIRST_PAGE=daily
#
CLEOF
fi

DISPLAY_FIRST_PAGE=`grep "^DISPLAY_FIRST_PAGE" $IPACC_CONF| tr -d " " | cut -d '=' -f 2`;

# tmp file
TMPFILE="/tmp/ipacc.txt";

# default don't read accounting information
READ_NEW_ACC=0;

# which day is to be indicated
if [ "$FORM_ACCMON" = "" ];then
	# today
	SHOW_ACC_MON=$ACC_MON;
	if [ "$DISPLAY_FIRST_PAGE" = "daily" ]; then
		SHOW_ACC_DAY="$ACC_DATE";
	else
		SHOW_ACC_DAY="";
	fi
	# read accounting
	READ_NEW_ACC=1;
elif [ "$FORM_ACCMON" = "$ACC_DATE" ]; then
        # today
        SHOW_ACC_MON=$ACC_MON;
        SHOW_ACC_DAY=$ACC_DATE;
        # read accounting
        READ_NEW_ACC=1;
else
	# day from cgi parameter (ACCMON=yyymm)
	SHOW_ACC_MON=$FORM_ACCMON;
	SHOW_ACC_DAY="";
fi

# display accounting for indicated ip
if [ "$FORM_ACCIP" != "" ]; then
	SHOW_ACC_IP="$FORM_ACCIP";
fi

# create new accounting chain because /etc/ipacc.conf is edit
NEW_ACC_IPS=0;
if [ "$FORM_NEWACCIPS" = "1" ]; then
	NEW_ACC_IPS="1";
fi

# end of variables
#####################################################################

#####################################################################
#
if [ "$FORM_EDITCONF" = "1" ]; then

	cl_header2 "Edit $CONFFILE";

	case "$FORM_OKBTN" in

		"$Feo")
        		echo "$Pja $IPACC_CONF";
                	echo "<pre>";
                	echo "$FORM_CFGFILE";
                	echo "</pre>";
                	echo "$FORM_CFGFILE" > $IPACC_CONF;
                	echo
     	           	echo "Save accounting and recreate accounting chain ...";
                	$RC_IPACC;
                	echo " done<br><br>";
        	        echo "<b><a href=backup.cgi><u>$Wqa</u></a></b><br>";
	                touch /tmp/need.save;
        	        ;;
		*)
			cat << CLEOF
			<form method="POST" action="/cgi-bin/ipacc.cgi?EDITCONF=1">
  			  <input type=hidden name=EDITCONF value="1">
  			  <table border="0">
    			    <tr>
      			      <td width="100%" align="center" bgcolor="#000080">
       			       <font color="#FFFFFF"><b>$IPACC_CONF</b></font>
                              </td>
                            </tr>
                            <tr>
                              <td width="100%"><textarea rows="19" name="CFGFILE" cols="80" wrap="off">
CLEOF
			cat $IPACC_CONF;
			cat << CLEOF
       		               </textarea>
                              </td>
    			    </tr>
    			    <tr>
      			      <td width="100%">
       			       <p align="center">
       			       <input type="submit" value="$Feo" name="OKBTN">
       			       <input type="reset" value="$Fer" name="B2">
      			      </td>
    			    </tr>
  			  </table>
			</form>
CLEOF
			;;
	esac
	cl_footer2;
	exit;
	
# read accounting information
# only read new accounting if ACCMON is not indicated
elif [ "$READ_NEW_ACC" = "1" ]; then
	# check, if accounting chain exist
	$IPTABLES -L $ACC_CHAIN -n 2>&1 >/dev/null;
	if [ $? -gt 0 ]; then
		cl_header2 $TITLE;
		echo "<center><b>";
		echo "Accounting chain '$ACC_CHAIN' not found !<br><br></b>";
		echo "try create $ACC_CHAIN with $RC_IPACC ...";
		if [ ! -e $RC_IPACC ];then
			echo "$RC_IPACC not found<br>";
			echo "$0 does not work<br>";
		else
			$RC_IPACC
			echo "$ACC_CHAIN created<br>";
			echo "<a href=$0><u>please reload $0</u></a><br>";
		fi
		echo "</center>";
		cl_footer2;
		exit;
	fi
	ACC_IPS=`$IPTABLES -L $ACC_CHAIN -n | grep -c RETURN`;
	CONF_ACC_IPS=`grep -c "^[0-9]\+.[0-9]\+.[0-9]\+.[0-9]\+" $IPACC_CONF`;
	
	if [ $ACC_IPS -lt 1 ]; then
		cl_header2 $TITLE;
		echo "<center><b>";
		echo "Accounting chain defunct. Recreate accounting chain ...<br></b>";
		$RC_IPACC;
		echo "<br>";
		echo "<a href=$0><u>please reload $0</u></a><br>";
		echo "</center>";
		cl_footer2;
		exit;
	fi

	# no ip's for accounting	
	if [ $ACC_IPS -eq 1 -a $CONF_ACC_IPS -eq 0 ]; then
		cl_header2 $TITLE;
		echo "<center><b>"; 
		echo "<b>No IP's for accounting found! Please edit <i>/ect/ipacc.conf!</b></i><br>";
		echo "<br>";
		echo "<br>";
		echo "<a href=ipacc.cgi?EDITCONF=1>[&nbsp;<u>Edit&nbsp;Configuration&nbsp;File</u>&nbsp;]</a><br>";
		echo "</center>";
		cl_footer2;
		exit;
	fi

	# show iptables chain accounting, delete double space, find IF_INET and 
	# output field 3 and write accounting into file
	$IPTABLES -Z -L $ACC_CHAIN -n -x -v | tr -s " " | grep $IF_INET | cut -d ' ' -f 3,9,10 > $TMPFILE 2>/dev/null;

	# rename accounting file
	if [ -e $ACC_FILE ]; then
		mv -f $ACC_FILE $ACC_FILE_OLD 
		# all ip's with trafficcounter is 0 
		# found ip's with values in accountingfile and 
		# no entry in acc-tmp-file (delete from ipacc.conf)
		for ACC_IP in `( grep -v "0.0.0.0\/0$" $TMPFILE | cut -d ' ' -f 3; cut -d ':' -f 1 $ACC_FILE_OLD ) | sort | uniq -u | grep -v "^#"`; do
			grep "$ACC_IP:" $ACC_FILE_OLD >> $ACC_FILE;
		done
			
	fi

	# read accounting information and write it to file 
	for ACC_IP in `grep "^[0-9]\+ 0.0.0.0\/0" $TMPFILE | cut -d ' ' -f 3`;do
		# found OLD OUT and IN traffic
       	 	OLD_ACC_OUT=`grep "^$ACC_IP:" $ACC_FILE_OLD | cut -d: -f2`;
       	 	OLD_ACC_IN=`grep "^$ACC_IP:" $ACC_FILE_OLD | cut -d: -f3`;
        	# find NEW OUT and IN traffic
        	ACC_OUT=`grep "$ACC_IP 0.0.0.0\/0$" $TMPFILE | tr -d "\n" | cut -d ' ' -f1`;
        	ACC_IN=`grep "0.0.0.0\/0 $ACC_IP$" $TMPFILE | tr -d "\n" | cut -d ' ' -f1`;

		# if variable zero, then set it to 0 
		if [ "$OLD_ACC_OUT" = "" ];then
 			OLD_ACC_OUT=0;
		fi
        	if [ "$OLD_ACC_IN" = "" ];then
        		OLD_ACC_IN=0;
        	fi
        	if [ "$ACC_OUT" = "" ];then
                	ACC_OUT=0;
       	 	fi
        	if [ "$ACC_IN" = "" ];then
                	ACC_IN=0;
        	fi
	                                                           
		# calculate
		ACC_OUT_SUM=`$ACCSUM -x $OLD_ACC_OUT $ACC_OUT`;
		ACC_IN_SUM=`$ACCSUM -x $OLD_ACC_IN $ACC_IN`;

		# write to acc file
		echo "$ACC_IP:$ACC_OUT_SUM:$ACC_IN_SUM" >> $ACC_FILE
	done
	
	# create note to need save
	touch /tmp/need.save

	# configuration has change (modify /etc/ipacc.conf)
        if [ $(($CONF_ACC_IPS*2)) -ne $((ACC_IPS-1)) ]; then
        	cl_header2 $TITLE;
                echo "<center><b>";
                echo "Configuration has been change! Accounting is saved!<br></b>";
                echo "Recreate accounting chain ...";
                $RC_IPACC;
                echo " done<br>";
                echo "<a href=$0><u>please reload $0</u></a><br>";
                echo "</center>";
                cl_footer2;
                exit
        fi
                                                                                                                                                                                
fi

# end of read accounting information 
#####################################################################

# if no output, then exit
if [ "$QUIET" = "1" ]; then
	exit
fi

#####################################################################
# beginn HTML output

# HTML header
cl_header2 $TITLE;
echo "<center>";

# display warning
if [ "$READ_NEW_ACC" = "0" -a "$SHOW_ACC_MON" = "$ACC_MON" ]; then
	echo "<b>Warning: ACCMON selected, don't read new accounting data</b><br>";
	echo "<a href=ipacc.cgi?ACCIP=$SHOW_ACC_IP>[&nbsp;<u>read new accountings</u>&nbsp;]</a><br>";
fi

# create new accounting chain
if [ "$NEW_ACC_IPS" = "1" ]; then
	$RC_IPACC;
	echo "<b>New accounting chain created.</b>";
	echo "Click <a href=ipacc.cgi>[&nbsp;<u>here</u>&nbsp;]</a> to reload page.";
	cl_footer2;
	exit;
fi

# HTML table 
echo "<table border=0 width=100%>";
echo " <tbody>";

# clear variables
ACC_OUT_SUM=0;
ACC_IN_SUM=0;

# list the accounting files
ACC_MONS=`ls -tr /var/spool/accounting/ipacc-????????.txt | sed -e 's/.*\/ipacc-\([0-9]\{6\}\)[0-9]\{2\}.txt/\1/' | sort -n | uniq`;

# HTML output
echo "  <form action="ipacc.cgi" method="get">";
echo "   <tr>";
echo "    <td colspan="8" rowspan="1" bgcolor="#000080" align=center>";
echo "     <font color=white><b>Show accounting for ";
if [ "$SHOW_ACC_IP" != "" ]; then
	echo "$SHOW_ACC_IP at";
fi

if [ ! -z "$SHOW_ACC_DAY" ]; then
	echo " day";
else
	echo " month";
fi

echo "&nbsp;<select name=\"ACCMON\">";
for ACC_MON in `echo $ACC_MONS`;do
	SELECTED="";
	if [ "$SHOW_ACC_MON" = "$ACC_MON" ]; then
 		SELECTED="selected";
 	fi
 	echo "<option value=\"$ACC_MON\" $SELECTED>$ACC_MON";
done

SELECTED="";
if [ ! -z "$SHOW_ACC_DAY" ]; then
	SELECTED="selected";
fi
echo "<option value=\"$ACC_DATE\" $SELECTED>$ACC_DATE";

echo "     </b></font>";
echo "     </select>";
echo "     <input type=\"submit\" value=\"submit\">";
echo "	   <input type=\"hidden\" name=\"ACCIP\" value=\"$SHOW_ACC_IP\"";
echo "    </td>";
echo "   </tr>";
echo "  </form>";
#echo " <tr>";
#echo "  <td>&nbsp;</td>";
#echo " </tr>";

for ACC_MON in `echo $ACC_MONS`;do
	# display detail accounting information for indicated IP for this month
        if [ "$SHOW_ACC_IP" != "" ]; then
        	if [ "$SHOW_ACC_MON" = "$ACC_MON" ]; then
       			# HTML output
        		ACC_CONVERT=0;
        		print_HTML "<b>Accounting Day</b>" "<b>Outgoing</b><br>(upload)&nbsp;" "<b>Incoming</b><br>(download)&nbsp;" "<b>Total</b><br>(outgoing and incoming)&nbsp;" "#C0C0C0";

			# find all accountings for this ip in ipacc-$ACC_MON??.txt		
			for LINE in `grep -H $SHOW_ACC_IP /var/spool/accounting/ipacc-$ACC_MON??.txt | tr -d " "`; do
				# find the accounting day from filename
				ACC_DAY=`echo $LINE | sed -e 's/.*\/ipacc-\([0-9]\{8\}\).txt:.*/\1/'`;
				# find accounting data
				ACC_OUT=`echo $LINE | cut -d ':' -f3`;
				ACC_IN=`echo $LINE | cut -d ':' -f4`;
				
				# if variable zero, then set it to 0
				if [ "$ACC_OUT" = "" ]; then
					ACC_OUT=0;
				fi
				if [ "$ACC_IN" = "" ]; then
					ACC_IN=0;
				fi
				
				# calculate
				ACC_SUM=`$ACCSUM -x $ACC_OUT $ACC_IN`;			
				ACC_SUM_OUT=`$ACCSUM -x $ACC_SUM_OUT $ACC_OUT`;
				ACC_SUM_IN=`$ACCSUM -x $ACC_SUM_IN $ACC_IN`;
				
				# HTML output
				
				ACC_CONVERT=1;
				USE_ACC_IP=0;
				USE_ACC_IP=`grep -c "^$SHOW_ACC_IP" $IPACC_CONF`;
				print_HTML $ACC_DAY $ACC_OUT $ACC_IN $ACC_SUM "#EDECEC";
			done
			
			# set variable to 0, if zero 
			# (SHOW_ACC_IP not found in accounting file)
			if [ -z "$ACC_SUM_IN" ]; then
				ACC_SUM_IN=0;
			fi
			if [ -z "$ACC_SUM_OUT" ]; then
				ACC_SUM_OUT=0;
			fi
									
			# calculate				
			ACC_MON_SUM=`$ACCSUM -x $ACC_SUM_IN $ACC_SUM_OUT`;

			# HTML output
			ACC_CONVERT=1;
			USE_ACC_IP=1;
			print_HTML "Summery for $ACC_MON" $ACC_SUM_OUT $ACC_SUM_IN $ACC_MON_SUM "#C0C0C0";
		fi;
	# display traffic for indicated month
        else 
        	if [ "$SHOW_ACC_MON" = "$ACC_MON" ]; then
       			# HTML output
       			ACC_CONVERT=0;
       	        	print_HTML "<b>Accounting IP</b>" "<b>Outgoing</b><br>(upload)&nbsp;" "<b>Incoming</b><br>(download)&nbsp;" "<b>Total</b><br>(outgoing and incoming)&nbsp;" "#C0C0C0";
			
			# find all ip's for accounting
                	for ACC_IP in `cut -d ':' -f1 /var/spool/accounting/ipacc-$ACC_MON??.txt | sed -e 's/\(.\+\)\.\([0-9]\+\)/\2:\1/' | sort -n | uniq | sed -e 's/\([0-9]\+\):\(.\+\)/\2.\1/'`;do
				# zero variables
				ACC_OUT_SUM=0;
				ACC_IN_SUM=0;
				ACC_SUM=0;
				USE_ACC_IP=0;
			
				# summery of traffic for this ip for this month
				if [ ! -z "SHOW_ACC_DAY" ]; then
					ACC_DAY="$SHOW_ACC_DAY";
				else
					ACC_DAY="$ACC_MON";
				fi
				ACC_OUT=`grep -H "$ACC_IP:" /var/spool/accounting/ipacc-$ACC_MON??.txt | grep "$ACC_DAY" | grep -v ":0:0"| cut -d ':' -f3 | tr "\n" " "`;
				ACC_IN=`grep -H "$ACC_IP:" /var/spool/accounting/ipacc-$ACC_MON??.txt | grep "$ACC_DAY" | grep -v ":0:0"| cut -d ':' -f4 | tr "\n" " "`;
	
				if [ -z "$ACC_OUT" ];then
					ACC_OUT_SUM="0 ";
				else
					# calculate sum of OUT
					ACC_OUT_SUM=`$ACCSUM -x $ACC_OUT`;
				fi
				if [ -z "$ACC_IN" ];then
					ACC_IN_SUM="0 ";
				else
					# calculate sum of IN
					ACC_IN_SUM=`$ACCSUM -x $ACC_IN`;
				fi
	
				# calculate
				ACC_SUM=`$ACCSUM -x $ACC_OUT_SUM $ACC_IN_SUM`;
				ACC_MON_OUT_SUM=`$ACCSUM -x $ACC_MON_OUT_SUM $ACC_OUT_SUM`;
				ACC_MON_IN_SUM=`$ACCSUM -x $ACC_MON_IN_SUM $ACC_IN_SUM`;

				# HTML output
				AHREF="ipacc.cgi?ACCMON=$ACC_MON&AMP;ACCIP=$ACC_IP";
				ACC_CONVERT=1;
				USE_ACC_IP=`grep -c "^$ACC_IP" $IPACC_CONF`;
				print_HTML $ACC_IP $ACC_OUT_SUM $ACC_IN_SUM $ACC_SUM "#EDECEC" $AHREF;
				
			done
			
			# calculate		
			ACC_MON_SUM=`$ACCSUM -x $ACC_MON_IN_SUM $ACC_MON_OUT_SUM`;
			
			# HTML output
			ACC_CONVERT=1;
			USE_ACC_IP=1;
			print_HTML "Summery of $ACC_MON" $ACC_MON_OUT_SUM $ACC_MON_IN_SUM $ACC_MON_SUM "#C0C0C0";
		fi		
	fi

done
echo "</select>";
echo "</form>";
                                
# HTML Output
echo " </tbody>";
echo "</table>";
echo "</center><b><small>*) these IP addresses are not any longer configure in <i>/etc/ipacc.conf</i></small></b><br><center>";
echo "<br>";
echo "<a href=ipacc.cgi?ACCMON=$ACC_MON>[&nbsp;<u>Back to Main</u>&nbsp;]</a>&nbsp;";
echo "<a href=ipacc.cgi?EDITCONF=1>[&nbsp;<u>Edit&nbsp;Configuration&nbsp;File</u>&nbsp;]</a>";
echo "</center>";
echo "<div style=\"text-align: right;\"><small>ipacc Version: $VERSION</small></div><br>";
cl_footer2;

rm -f $TMPFILE;
rm -f $ACC_FILE_OLD

# end of HTML output
#####################################################################
